#include "BSL_UART_Protocol_5xx.h"
#include "BSL_Command_Definitions_5xx.h"
#include "BSL_IO_UART.h"
#include "BSL_Definitions.h"
#include "CRC.h"

/*******************************************************************************
*Function:     UART_5xx_RXPacket
*Description:  reads data via UART communication
*Parameters:
*              dataBuffer *db       a struct  where read data will be stored
*Returns:
*              unsigned char        FLASH_ACK if operation was successfull
*******************************************************************************/
unsigned char UART_5xx_RXPacket(dataBuffer *db)
{
	unsigned int i;
	unsigned char scrap_var;
	unsigned char RX_checksum_low = 0x00;
	unsigned char RX_checksum_high = 0x00;
	unsigned char RX_header;

	if (UART_GetVerbose() == 1)
	{
		printf("\n-------------------------------------------------\n");
	}
	RX_header = readByte();
/*---------------------------------------HEADER-------------------------*/
	if (RX_header == TIMEOUT_ERROR)
	{
#ifdef DEBUG_ERRORS
		printf( "\n---------------Timeout on header----------------\n");
#endif
		ReadGargbageUntilTimeout();
		return (BYTE)TIMEOUT_ERROR;
	}
	else if (RX_header != (BYTE)0x80)
	{
#ifdef DEBUG_ERRORS
		printf( "\n---------------Header != 0x80-------------------\n");
#endif
		ReadGargbageUntilTimeout();
		return HEADER_INCORRECT;
	}
/*---------------------------------------Size 1-------------------------*/
	scrap_var = readByte();
	if (scrap_var == TIMEOUT_ERROR)
	{
#ifdef DEBUG_ERRORS
		printf( "\n---------------Timeout on rx size(1)------------\n");
#endif
		ReadGargbageUntilTimeout();
		return (BYTE)TIMEOUT_ERROR;
	}
	db->size = scrap_var;
/*---------------------------------------Size 2-------------------------*/
	scrap_var = readByte();
	if (scrap_var == TIMEOUT_ERROR)
	{
#ifdef DEBUG_ERRORS
		printf( "\n---------------Timeout on rx size(2)------------\n");
#endif
		ReadGargbageUntilTimeout();
		return (BYTE)TIMEOUT_ERROR;
	}

	db->size |= (((DWORD)scrap_var) << 8);
	Flash_crcInit(0xFFFF);
/*---------------------------------------Reading Data-------------------------*/
	for (i = 0; i < db->size; i++)
	{
		db->data[i] = (BYTE)readByte();
		Flash_crcInput((BYTE)db->data[i]);
	}
/*---------------------------------------Checksum low-------------------------*/
	RX_checksum_low = readByte();
	if (RX_checksum_low != Flash_getLowByte())
	{
		printf("    Fail on low byte, expected:\t%2.2X recieved:\t%2.2X\n", (Flash_getLowByte() & 0xFF),(RX_checksum_low & 0xFF));
		ReadGargbageUntilTimeout();
		return CHECKSUM_INCORRECT;
	}
/*---------------------------------------Checksum high-------------------------*/
	RX_checksum_high = readByte();
	if (RX_checksum_high != Flash_getHighByte())
	{
		printf("    Fail on high byte, expected:\t%2.2X recieved:\t%2.2X\n", (Flash_getHighByte() & 0xFF),(RX_checksum_high & 0xFF));
		ReadGargbageUntilTimeout();
		return CHECKSUM_INCORRECT;
	}
	if (UART_GetVerbose() == 1)
	{
		printf("\n-------------------------------------------------\n");
	}
	return FLASH_ACK;
}

/*******************************************************************************
*Function:    UART_5xx_TXPacket
*Description: writes data via UART communication
*Parameters:
*             dataBuffer db     struct with data that will be written to device
*Returns:
*             unsigned char     FLASH_ACK if operation was successfull
*******************************************************************************/
unsigned char UART_5xx_TXPacket(dataBuffer db)
{
	unsigned int i = 0;
	unsigned int x = 0;
	unsigned char answer = 0;
	unsigned char data [MAX_BUFFER_SIZE + 5] = {0}; 

	Flash_crcInit(0xFFFF);
	if (UART_GetVerbose() == 1)
	{
		printf("\n-------------------------------------------------\n");
	}
	data[i++] = ((unsigned char)0x80);
	data[i++] = ((unsigned char)(db.size & 0xFF));
	data[i++] = ((unsigned char)(db.size >> 8 & 0xFF));

	for(x = 0; x < db.size; i++)
	{
		data[i] = db.data[x];
		Flash_crcInput(db.data[x]);
		x++;
	}
	data[i++] = Flash_getLowByte(); 
	data[i++] = Flash_getHighByte();

	writeByte(data, db.size + 5);
	answer = readByte();
	if (answer != FLASH_ACK)
	{
		clearBuffers();
	}
	if (UART_GetVerbose() == 1)
	{
		printf("\n-------------------------------------------------\n");
	}
	return answer;
}
